@@ -1,6 +1,7 @@
 from sanic.app import Sanic
 from sanic.blueprints import Blueprint
 
-__version__ = '0.8.3'
 
-__all__ = ['Sanic', 'Blueprint']
+__version__ = "18.12.0"
+
+__all__ = ["Sanic", "Blueprint"]
